#include <PID_v1.h>    // Controller LIB
#include <Encoder.h>   // Winkelencoder LIB


//Mechanical Sytem Properties
  //full pendulum rotation =8000 Encoder counts
#define rot_Encoder_A 20    // rotary Encoderpin A an Pin 20
#define rot_Encoder_B 21    // rotary Encoderpin B an Pin 21
#define lin_Encoder_A 18    // linear Encoderpin A an Pin 18
#define lin_Encoder_B 19    // linear Encoderpin B an Pin 19
Encoder rotEnc(rot_Encoder_A, rot_Encoder_B);   
Encoder linEnc(lin_Encoder_A, lin_Encoder_B);   

//Variable Definitions

  //Pin Definitions
   byte pin_photoint=13;
   int RPWM_Output = 6; // Arduino PWM output pin 6; connect to IBT-2 pin 1 (RPWM)
   int LPWM_Output = 7; // Arduino PWM output pin 7; connect to IBT-2 pin 2 (LPWM)
 
 //Linear and angle Limits for Controller
    long limit_lin_min= 1600;    //min Limit (in direction torwards limit switch--> right)
    long limit_lin_max= 40000;   //max Limit (left)
    double limit_angle= 200; //limit at which it becomes impossible to catch the pendulum again if exeeded, active control of pendulum is stopped and cart is stopped if exeeded. Limit works in both direction (+ and -) at the equilibrium point
   
 //Deadband Variables
     byte output_deadband= 30;
 
    
  //Miscellaneous Variables
    int rot_offset=0;   //Offset from Vertical, System inherent error. Use if System always tends to run in one direction
    double angle;
    double lin_position;
 
  
//PID Setup
  //PID Variables 
    double rot_kp= 8.2;
    double rot_ki= 104.0;
    double rot_kd= 0.0;
    double lin_kp= 0.00012; 
    double lin_ki= 0.0000005; 
    double lin_kd= 0.0;
    double Output, rot_Setpoint;
    double lin_Setpoint = 21000; // 21000 is approx. middle of track
         
 //PID Controller initialisation
    PID rot_Controller(&angle,&Output,&rot_Setpoint,rot_kp,rot_ki,rot_kd,DIRECT);
    PID lin_Controller(&lin_position,&rot_Setpoint,&lin_Setpoint,lin_kp,lin_ki,lin_kd,DIRECT); 
 
void setup() {
  // PWM Frequenz fuer Timer 4 auf 31kHz setzen
  int myEraser = 7;
  TCCR4B &= ~myEraser;  // Timerregister fuer timer 4 loeschen
  int myPrescaler = 1;  // Vorteiler definieren       
  TCCR4B |= myPrescaler;  // Vorteiler laden
  // Ein / Ausgänge definieren
  pinMode(RPWM_Output, OUTPUT);
  pinMode(LPWM_Output, OUTPUT);
  analogWrite(LPWM_Output, 0);
  analogWrite(RPWM_Output, 0);  
  Serial.begin(115200); 
  pinMode(pin_photoint,INPUT);
 
//Initialize Encoder counts
  rotEnc.write(-4000); //set when pendulum is hanging straight down due to gravity
  linEnc.write(lin_Setpoint);  
 //PID Settings
     rot_Controller.SetMode(AUTOMATIC);
     rot_Controller.SetOutputLimits(-250,250); 
     rot_Controller.SetSampleTime(4); //2
     rot_Controller.SetTunings(rot_kp,rot_ki,rot_kd);
     
     lin_Controller.SetMode(AUTOMATIC);
     lin_Controller.SetOutputLimits(-300,300); 
     lin_Controller.SetSampleTime(4); //2
     lin_Controller.SetTunings(lin_kp,lin_ki,lin_kd);
}

void loop() {
  angle = rotEnc.read();
  lin_position = linEnc.read();
  if ((lin_position>limit_lin_min) && (lin_position<limit_lin_max)){       //perform as long carriage is within the linear limits  
            if (angle < limit_angle && angle > (limit_angle*-1)){    //perform as long as pendulum is within the angular limits  
                                 //Execute linear PID controller
                                 lin_Controller.Compute();
                                 // Serial.println(rot_Setpoint);   // bei Bedarf Ausgabe des Setpoints auf dem seriellen Plotter
                                 rot_Setpoint = rot_Setpoint + rot_offset;
                                 // Execute angular PID controller
                                 rot_Controller.Compute();
                                 Serial.println(Output);        // Ausgabe der Stellgroesse auf dem seriellen Plotter   
                                 if(Output > output_deadband){
                                     analogWrite(LPWM_Output, 0);
                                     analogWrite(RPWM_Output, Output); 
                                 }     
                                 if(Output < -output_deadband){
                                    analogWrite(RPWM_Output, 0);
                                    analogWrite(LPWM_Output, abs(Output));
                                 }
                                 if(Output<output_deadband&&Output>-output_deadband){
                                    analogWrite(LPWM_Output, 0);
                                    analogWrite(RPWM_Output, 0);
                                    // Output=0;
                                }
                      
                    } //stop carrigage if pendulum has tipped out of the angular limits
              else {   
                          analogWrite(LPWM_Output, 0);
                          analogWrite(RPWM_Output, 0);    
                    };
              }  //stop carrigage out of the linear limits
     else {
             analogWrite(LPWM_Output, 0);
             analogWrite(RPWM_Output, 0);
    };    
 } //End Loop
 
